package yrpg.bamp.basedata.plugin.other.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.exception.KDException;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.util.HttpClientUtils;
import kd.bos.util.StringUtils;
import kd.sdk.plugin.Plugin;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 后台任务插件
 */
public class PushAdminDivisionToMDMTask extends AbstractTask implements Plugin {

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {

        String url=System.getProperty("yrpg.fi.mdm.areainsert.url");

        Map<String,String> header =new HashMap<>();
        header.put("Content-Type","application/json;charset=utf-8");

        //推送国家和地区
        QFilter[] countryfilters = new QFilter[]{};
        DynamicObjectCollection countrys= QueryServiceHelper.query("bd_country","id,number,name,description,simplespell,areacode,enable,timezone.number,timezone.name",countryfilters);
        if(countrys.size()!=0){
            JSONObject paramsLogin = new JSONObject();
            JSONArray masterData = new JSONArray();
            for(DynamicObject country:countrys){
                JSONObject data = new JSONObject();
                long countryid=country.getLong("id");//id
                String number="C"+country.getString("number");//编码
                String name=country.getString("name");//名称
                String fullspell=country.getString("description");//英文名称
                String simplespell=country.getString("simplespell");//英文简称
                String level="0";//层级
                Boolean isleaf=false;//是否末级
                String isleafname=null;
                //String parentnumber="C"+country.getString("number");//上级区划
                String citynumber=country.getString("areacode");//电话区号
                String countryname=country.getString("name");//所属国家
                String longnumber="C"+country.getString("number");//长编码
                String timezonenumber=country.getString("timezone.number");//时区编码
                String timezonename=country.getString("timezone.name");//时区名称
                String enable=country.getString("enable");//启用状态  0:停用   1:启用
                String divtypecode="GJ001";//类型编码
                String divtype="国家或地区";//类型名称
                if("1".equals(enable)){
                    enable="启用";
                }else{
                    enable="停用";
                }

                if(isleaf){
                    isleafname="是";
                }else{
                    isleafname="不是";
                }

                if(timezonenumber==null){
                    timezonenumber="";
                }
                if(timezonename==null){
                    timezonename="";
                }

                data.put("id",String.valueOf(countryid));
                data.put("code",number);
                data.put("name",name);
                data.put("name_en",fullspell);
                data.put("shortname_en",simplespell);
                data.put("div_level",level);
                data.put("isleaf#name",isleafname);
                //data.put("parentcode",parentnumber);
                data.put("tel_code",citynumber);
                data.put("country_name",countryname);
                data.put("long_code",longnumber);
                data.put("time_zone_code",timezonenumber);
                data.put("time_zone_name",timezonename);
                data.put("enablestate#name",enable);
                data.put("div_type_code",divtypecode);
                data.put("div_type",divtype);

                masterData.add(data);
            }

            paramsLogin.put("masterData",masterData);
            String reponseStrLogin="";

            Map<String, Object> logMap = new HashMap<>();
            logMap.put("opname", "定时任务推送国家和地区到MDM");
            logMap.put("thirdappname", "MDM");
            logMap.put("opdesc",url);
            logMap.put("request", paramsLogin.toString());
            logMap.put("status", "true");
            logMap.put("username", "yy");

            try{
                reponseStrLogin= HttpClientUtils.postjson(url,header,paramsLogin.toString(),20000,40000);
            }catch (Exception e){
                logMap.put("response", "调用MDM推送国家和地区接口失败！"+e.getMessage());
                logMap.put("status", "false");
                saveApiLog(logMap);
            }

            logMap.put("response",reponseStrLogin);
            if (!StringUtils.isEmpty(reponseStrLogin)) {
                JSONObject responseJSON=null;
                try {
                    responseJSON = JSONObject.parseObject(reponseStrLogin);
                }catch (Exception e){
                    logMap.put("status", "false");
                    long id=saveApiLog(logMap);

                    saveDetailApiLog(logMap,id);
                }
                if(responseJSON.get("success")==null){
                    long id=saveApiLog(logMap);
                    saveDetailApiLog(logMap,id);
                }
                if (Boolean.valueOf(responseJSON.get("success").toString())) {
                    logMap.put("response","推送推送国家和地区至主数据成功！");
                    saveApiLog(logMap);
                }else{
                    logMap.put("status", "false");
                    logMap.put("response", "调用MDM推送推送国家和地区接口返回异常！错误信息是"+responseJSON.get("message").toString());
                    long id=saveApiLog(logMap);
                    saveDetailApiLog(logMap,id);
                }
            } else {
                logMap.put("status", "false");
                logMap.put("response", "接口返回值为空");
                long id=saveApiLog(logMap);
                saveDetailApiLog(logMap,id);
            }
        }

        //推送行政区划
        QFilter[] filters = new QFilter[]{};
        DynamicObjectCollection admindivisions= QueryServiceHelper.query("bd_admindivision","id,number,name,fullspell,simplespell,level,isleaf,parent.number,parent.name,citynumber,enable,country.name,country.number,longnumber,timezone.number,timezone.name,basedatafield.number,basedatafield.name",filters,"level");

        //记录2级行政区划的编码及其上级名称
        Map<String, String> level2Map = new HashMap<>();
        QFilter lev2 = new QFilter("level", QCP.equals,2);
        QFilter[] lev2filters = new QFilter[]{lev2};
        DynamicObjectCollection admindivisionlev2s= QueryServiceHelper.query("bd_admindivision","number,parent.name",lev2filters);
        if(admindivisionlev2s.size()!=0){
            for(DynamicObject admindivisionlev2:admindivisionlev2s){
                level2Map.put(admindivisionlev2.getString("number"),admindivisionlev2.getString("parent.name"));
            }
        }

        //记录3级行政区划的编码及上级名称、上级编码
        Map<String, String> level3Map = new HashMap<>();

        JSONObject paramsLogin = new JSONObject();
        if(admindivisions.size()!=0){

            //分批推送，一次500条
            int count=admindivisions.size();
            int num=count/500;
            if(count%500!=0) {
                num++;
            }
            for(int i=0;i<num;i++){
                JSONArray masterData = new JSONArray();
                if(i==num-1){
                    for(int j=500*i;j<admindivisions.size();j++){
                        DynamicObject admindivision=admindivisions.get(j);
                        JSONObject data = new JSONObject();
                        long admindivisionid=admindivision.getLong("id");//id
                        String number=admindivision.getString("number");//编码
                        String name=admindivision.getString("name");//名称
                        String fullspell=admindivision.getString("fullspell");//英文名称
                        String simplespell=admindivision.getString("simplespell");//英文简称
                        String level=admindivision.getString("level");//层级
                        Boolean isleaf=admindivision.getBoolean("isleaf");//是否末级
                        String isleafname=null;
                        String parentnumber=admindivision.getString("parent.number");//上级区划
                        if(parentnumber==null||"".equals(parentnumber)){
                            parentnumber="C"+admindivision.getString("country.number");//国家
                        }
                        String citynumber=admindivision.getString("citynumber");//电话区号
                        String countryname=admindivision.getString("country.name");//所属国家
                        String longnumber=admindivision.getString("longnumber");//长编码
                        String timezonenumber=admindivision.getString("timezone.number");//时区编码
                        String timezonename=admindivision.getString("timezone.name");//时区名称
                        String enable=admindivision.getString("enable");//启用状态  0:停用   1:启用
                        String divtypecode=admindivision.getString("basedatafield.number");//类型编码
                        String divtype=admindivision.getString("basedatafield.name");//类型名称

                        String countyname="";//所在区县名称
                        String cityname="";//所在城市名称
                        String provincename="";//所在省份名称

                        if("1".equals(level)){
                            provincename=name;
                        }else if("2".equals(level)){
                            cityname=name;
                            provincename=admindivision.getString("parent.name");//上级名称
                        }else if("3".equals(level)){
                            countyname=name;
                            cityname=admindivision.getString("parent.name");//上级名称
                            provincename=level2Map.get(parentnumber).toString();//获取1级名称
                            String numandnamelv2=parentnumber+"&"+admindivision.getString("parent.name");
                            level3Map.put(number,numandnamelv2);
                        }else if("4".equals(level)){
                            countyname=admindivision.getString("parent.name");//3级名称
                            String numandnamelv2=level3Map.get(parentnumber).toString();//2级编码/名称
                            cityname=numandnamelv2.split("&")[1];//2级名称
                            provincename=level2Map.get(numandnamelv2.split("&")[0]).toString();//1级名称
                        }

                        if("1".equals(enable)){
                            enable="启用";
                        }else{
                            enable="停用";
                        }

                        if(isleaf){
                            isleafname="是";
                        }else{
                            isleafname="不是";
                        }

                        if(parentnumber==null){
                            parentnumber="";
                        }
                        if(countryname==null){
                            countryname="";
                        }
                        if(timezonenumber==null){
                            timezonenumber="";
                        }
                        if(timezonename==null){
                            timezonename="";
                        }
                        if(divtypecode==null){
                            divtypecode="";
                        }
                        if(divtype==null){
                            divtype="";
                        }

                        data.put("id",String.valueOf(admindivisionid));
                        data.put("code",number);
                        data.put("name",name);
                        data.put("name_en",fullspell);
                        data.put("shortname_en",simplespell);
                        data.put("div_level",level);
                        data.put("isleaf#name",isleafname);
                        data.put("parentcode",parentnumber);
                        data.put("tel_code",citynumber);
                        data.put("country_name",countryname);
                        data.put("long_code",longnumber);
                        data.put("time_zone_code",timezonenumber);
                        data.put("time_zone_name",timezonename);
                        data.put("enablestate#name",enable);
                        data.put("div_type_code",divtypecode);
                        data.put("div_type",divtype);

                        data.put("province_name",provincename);
                        data.put("city_name",cityname);
                        data.put("county_name",countyname);

                        masterData.add(data);
                    }
                }else{
                    for(int j=500*i;j<500*(i+1);j++){
                        DynamicObject admindivision=admindivisions.get(j);
                        JSONObject data = new JSONObject();
                        long admindivisionid=admindivision.getLong("id");//id
                        String number=admindivision.getString("number");//编码
                        String name=admindivision.getString("name");//名称
                        String fullspell=admindivision.getString("fullspell");//英文名称
                        String simplespell=admindivision.getString("simplespell");//英文简称
                        String level=admindivision.getString("level");//层级
                        Boolean isleaf=admindivision.getBoolean("isleaf");//是否末级
                        String isleafname=null;
                        String parentnumber=admindivision.getString("parent.number");//上级区划
                        if(parentnumber==null||"".equals(parentnumber)){
                            parentnumber="C"+admindivision.getString("country.number");//国家
                        }
                        String citynumber=admindivision.getString("citynumber");//电话区号
                        String countryname=admindivision.getString("country.name");//所属国家
                        String longnumber=admindivision.getString("longnumber");//长编码
                        String timezonenumber=admindivision.getString("timezone.number");//时区编码
                        String timezonename=admindivision.getString("timezone.name");//时区名称
                        String enable=admindivision.getString("enable");//启用状态  0:停用   1:启用
                        String divtypecode=admindivision.getString("basedatafield.number");//类型编码
                        String divtype=admindivision.getString("basedatafield.name");//类型名称

                        String countyname="";//所在区县名称
                        String cityname="";//所在城市名称
                        String provincename="";//所在省份名称

                        if("1".equals(level)){
                            provincename=name;
                        }else if("2".equals(level)){
                            cityname=name;
                            provincename=admindivision.getString("parent.name");//上级名称
                        }else if("3".equals(level)){
                            countyname=name;
                            cityname=admindivision.getString("parent.name");//上级名称
                            provincename=level2Map.get(parentnumber).toString();//获取1级名称
                            String numandnamelv2=parentnumber+"&"+admindivision.getString("parent.name");
                            level3Map.put(number,numandnamelv2);
                        }else if("4".equals(level)){
                            countyname=admindivision.getString("parent.name");//3级名称
                            String numandnamelv2=level3Map.get(parentnumber).toString();//2级编码/名称
                            cityname=numandnamelv2.split("&")[1];//2级名称
                            provincename=level2Map.get(numandnamelv2.split("&")[0]).toString();//1级名称
                        }

                        if("1".equals(enable)){
                            enable="启用";
                        }else{
                            enable="停用";
                        }

                        if(isleaf){
                            isleafname="是";
                        }else{
                            isleafname="不是";
                        }

                        if(parentnumber==null){
                            parentnumber="";
                        }
                        if(countryname==null){
                            countryname="";
                        }
                        if(timezonenumber==null){
                            timezonenumber="";
                        }
                        if(timezonename==null){
                            timezonename="";
                        }
                        if(divtypecode==null){
                            divtypecode="";
                        }
                        if(divtype==null){
                            divtype="";
                        }

                        data.put("id",String.valueOf(admindivisionid));
                        data.put("code",number);
                        data.put("name",name);
                        data.put("name_en",fullspell);
                        data.put("shortname_en",simplespell);
                        data.put("div_level",level);
                        data.put("isleaf#name",isleafname);
                        data.put("parentcode",parentnumber);
                        data.put("tel_code",citynumber);
                        data.put("country_name",countryname);
                        data.put("long_code",longnumber);
                        data.put("time_zone_code",timezonenumber);
                        data.put("time_zone_name",timezonename);
                        data.put("enablestate#name",enable);
                        data.put("div_type_code",divtypecode);
                        data.put("div_type",divtype);

                        data.put("province_name",provincename);
                        data.put("city_name",cityname);
                        data.put("county_name",countyname);

                        masterData.add(data);
                    }
                }

                paramsLogin.put("masterData",masterData);
                String reponseStrLogin="";

                Map<String, Object> logMap = new HashMap<>();
                logMap.put("opname", "定时任务推送行政区划到MDM");
                logMap.put("thirdappname", "MDM");
                logMap.put("opdesc",url);
                logMap.put("request", paramsLogin.toString());
                logMap.put("status", "true");
                logMap.put("username", "yy");

                try{
                    reponseStrLogin= HttpClientUtils.postjson(url,header,paramsLogin.toString(),20000,40000);
                }catch (Exception e){
                    logMap.put("response", "调用MDM推送行政区划接口失败！"+e.getMessage());
                    logMap.put("status", "false");
                    saveDetailApiLog(logMap,saveApiLog(logMap));
                }

                logMap.put("response",reponseStrLogin);
                if (!StringUtils.isEmpty(reponseStrLogin)) {
                    JSONObject responseJSON=null;
                    try {
                        responseJSON = JSONObject.parseObject(reponseStrLogin);
                    }catch (Exception e){
                        logMap.put("status", "false");
                        saveDetailApiLog(logMap,saveApiLog(logMap));
                    }

                    if (Boolean.valueOf(responseJSON.get("success").toString())) {
                        logMap.put("response","推送行政区划至主数据成功！");
                        saveApiLog(logMap);
                    }else{
                        logMap.put("status", "false");
                        logMap.put("response", "调用MDM推送行政区划接口返回异常！错误信息是"+responseJSON.get("message").toString());

                        saveDetailApiLog(logMap,saveApiLog(logMap));
                    }
                } else {
                    logMap.put("status", "false");
                    logMap.put("response", "接口返回值为空");

                    saveDetailApiLog(logMap,saveApiLog(logMap));
                }
            }
        }

    }

    public static long saveApiLog(Map<String,Object> logMap){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_data");
        ORM.create().setPrimaryKey(apiLog);
        //api编号
        apiLog.set("opname",logMap.get("opname"));
        //URL
        apiLog.set("opdesc",logMap.get("opdesc"));
        //请求参数
        if(logMap.get("request")!=null){
            apiLog.set("request",logMap.get("request").toString().length()>2000?logMap.get("request").toString().substring(0, 2000):logMap.get("request"));
        }else {
            apiLog.set("request","");
        }

        if (logMap.get("response")!=null){
            //响应结果
            apiLog.set("response",logMap.get("response").toString().length()>2000?logMap.get("response").toString().substring(0, 2000):logMap.get("response"));
        }else {
            apiLog.set("response","");
        }
        //调用时间
        apiLog.set("opdate",new Date());
        //调用者
        apiLog.set("username",logMap.get("username"));
        //第三方应用
        apiLog.set("thirdappname",logMap.get("thirdappname"));
        //调用状态
        apiLog.set("status",logMap.get("status"));
        //保存日志
        SaveServiceHelper.save(new DynamicObject[]{apiLog});

        return apiLog.getLong("id");

    }

    public static void saveDetailApiLog(Map<String,Object> logMap,long id){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_detail");
        //调用者
        apiLog.set("username",logMap.get("username"));
        //调用时间
        apiLog.set("opdate",new Date());
        //入参
        apiLog.set("reqpara_tag",logMap.get("request").toString());
        //响应
        apiLog.set("resppara_tag",logMap.get("response").toString());

        apiLog.set("id",id);

        SaveServiceHelper.save(new DynamicObject[]{apiLog});

    }
}